// -*- C++ -*-
//
//  This file is part of WEEVILGEN.
//
//  Authors: See the file AUTHORS for full details.
//
//  Copyright (C) 2002-2004 University of Colorado
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
//  USA, or send email to serl@cs.colorado.edu.
//
// $Id: analysisbrowser.cc,v 1.1 2007-07-10 13:08:07 wolf Exp $
//
#include <cassert>
#include <cstring>
#include <string>
#include <weevil/weevil.h>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>

using namespace ssim;
using namespace std;
using namespace weevil;

/**
 * application-specific function and classes
 */
unsigned int get_rand( unsigned int a, unsigned int b )
{
  // We use the following formula to scale rand() to
  // the range of values we are looking for a -> b inclusive:
  // x = a + int( (rand() / (RAND_MAX+1.0))*(b-a+1) )
  return( a + (unsigned int)(((double)rand()/(RAND_MAX+1.0))*(double)(b-a\
								      +1)) );
}

/**
 * a simple sequential process class.  Producer holds the
 * id of a neighbor process.
 */
class AnalysisBrowser : public weevil::WeevilTProcess {
  /**
   * gsl
   */
  gsl_rng * gsl_r;
  const gsl_rng_type * gsl_T;
  /**
   * inputted properties from m4 configuration
   */
  unsigned int m_num_gets;
  unsigned int m_num_files;
  std::string m_distribution;
  double m_mean;

public:
  /**
   * sequential process, thus no implementation for "process_event" or "init"
   */
  virtual void main();

  /**
   * assignment of process properties
   */
  void num_gets( unsigned int n )
  { m_num_gets = n; }
  
  void num_files ( unsigned int n )
  { m_num_files = n; }

  void distribution ( std::string arg ){
    int begin = 0;
    int end = arg.find_first_of(" ", begin);
    m_distribution = arg.substr(begin, end-begin);
    begin = end + 1;
    if (m_distribution.compare("poisson") == 0)
      m_mean = atof(arg.substr(begin).c_str());
  }
};

/**
 * method implementation
 */
void AnalysisBrowser::main() {
    gsl_rng_env_setup();
    gsl_T = gsl_rng_default;
    gsl_r = gsl_rng_alloc(gsl_T);
    if (m_distribution.compare("poisson") == 0){
      for(int i = 0; i < m_num_gets; ++i) {
	//
	// get poisson distribution
	//
	unsigned int m_get_timeout = gsl_ran_poisson(gsl_r, m_mean);
        //
        // the producer sleeps for some time here
        //
        WeevilGen::self_signal_event(NULL, m_get_timeout);
      }
    }
    for (int i = 0; i < m_num_gets; ++i) {
        //
        // the event we wait for here is the timeout we just set
        //
        wait_for_event();
        //
        // when we wake up after the timeout, we signal an event E to
        // our neighbor Consumer
        //
	unsigned int fid = get_rand( 1, m_num_files );
	char msg[256];
	sprintf( msg, "GET(file%d)", fid );
	workload_output( msg );
    }
}
