// -*- C++ -*-
//
//  This file is part of WEEVILGEN.
//
//  Authors: See the file AUTHORS for full details.
//
//  Copyright (C) 2002-2004 University of Colorado
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
//  USA, or send email to serl@cs.colorado.edu.
//
// $Id: producer.cc,v 1.1 2007-07-10 13:08:07 wolf Exp $
//
#include <cassert>
#include <cstring>
#include <string>
#include <weevil/weevil.h>

using namespace ssim;
using namespace std;
using namespace weevil;

/**
 * a simple sequential process class.  Producer holds the
 * id of a neighbor process.
 */
class Producer : public weevil::WeevilTProcess {
  /**
   * inputted properties from m4 configuration
   */
  std::string m_neighbor;
  unsigned int m_num_gets;

public:
  /**
   * sequential process, thus no implementation for "process_event" or "init"
   */
  virtual void main();

  /**
   * assignment of process properties
   */
  void num_gets( unsigned int n )
  { m_num_gets = n; }

  void neighbor( std::string s )
  { m_neighbor = s; }
};

/**
 * method implementation
 */
void Producer::main() {
    for(int i = 0; i < m_num_gets; ++i) {
        //
        // the producer sleeps for some time here (100 + i time units)
        //
        WeevilGen::self_signal_event(NULL, 100 + i);
        //
        // the event we wait for here is the timeout we just set
        //
        wait_for_event();
        //
        // when we wake up after the timeout, we signal an event E to
        // our neighbor Consumer
        //
	char msg[256];
	sprintf( msg, "Signal(%d)", i );
	workload_output( msg );
        WeevilGen::signal_event(m_neighbor, new E(i));
    }
}
