// -*-Java-*-
//
//  This file is part of WEEVIL.
//
//  Authors: See the file AUTHORS for full details.
//
//  Copyright (C) 2002-2006 University of Colorado
//
//  Weevil is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  Weevil is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
//  USA, or send email to serl@cs.colorado.edu.
//
package weevil;
import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

/** a generic virtual time workload controller
 *
 *	This class implements a generic virtual time controller which will
 *	control the scheduling of the workload execution according to the
 *	workload file.
 *
 */
 public class WeevilJavaActor{

	/** the overall controller of the whole execution process
	 *
	 *	This method control the time to start the execution, and also
	 *	read the workload file from the beginning to the end and call
	 *	the corresponding action for each line of the workload file.
	 *
	 *	In your system specific client workload actor, you should call
	 *	the run_simulation() method to start the whole process, and the
	 *	stop_simulation() method to stop the whole process.
	 *	For example, @see doc-files/Actor.java
	 */
	public static void waitAction(String arg){
		//////////////parsing////////////////
		int time = Integer.parseInt(arg);
		//////////////client API////////////////
		try {
			Thread.sleep(time);	// sleeps
		} catch (java.lang.InterruptedException ex) {
			System.out.println("interrupted");
		}
	}

	static void runSimulation(Actor dr, long t, String wkldfile){
	        //long currtime;
	        //String s_currtime;
	 	////////////time control////////////
		//Date currdate = new Date();
	 	//currtime = currdate.getTime();
	   	//long sleeptime = 1000*t-currtime;
		try {
			Thread.sleep(t);
		} catch (java.lang.InterruptedException ex) {
			System.out.println("interrupted");
		}

	 	////////////begin to read file////////////
		String thisLine;
		try{
			FileInputStream fin = new FileInputStream(wkldfile);
	 		BufferedReader myInput = new BufferedReader(new InputStreamReader(fin));
			while ((thisLine = myInput.readLine())!= null){
				if (thisLine.length() != 0){
					if (thisLine.substring(0, 1).equals("/")){
					}
					else if (thisLine.length() >= 9){
					  if (thisLine.substring(0, 9).equals("WVL_Wait(")){
						waitAction(thisLine.substring(9, thisLine.indexOf(")")));
					  }
					  else dr.action(thisLine);
					}
					else {
						dr.action(thisLine);
					}
				}
			}
			stopSimulation(dr);
		} catch (Exception ex) {
			ex.printStackTrace();
	    }
	}

	/** stopping the whole execution
	 *
	 *	This method asks the client to stop.
	 **/
	static void stopSimulation(Actor dr){
		dr.stop();
	}

	public static Object createObject(Constructor constructor,
					  Object[] arguments) {
	    
	    System.out.println ("Constructor: " + constructor.toString());
	    Object object = null;
	    
	    try {
		object = constructor.newInstance(arguments);
		System.out.println ("Object: " + object.toString());
		return object;
	    } catch (InstantiationException e) {
		System.out.println(e);
	    } catch (IllegalAccessException e) {
		System.out.println(e);
	    } catch (IllegalArgumentException e) {
		System.out.println(e);
	    } catch (InvocationTargetException e) {
		System.out.println(e);
	    }
	    return object;
	}
     
     public static void main(String[] args)
     {
	 /** per-client workload */
	 String workloadfile;
	 /** time to start the execution */
		long time;

		if(args.length != 4) {
	    	System.err.println("Usage: WeevilJavaActor <initial delay> <filename> <userclass name> <userclass configuration>");
	    	System.exit(1);
		}
		try{
			time = Integer.parseInt(args[0]);
			workloadfile = args[1];

			Actor actor;
			Class actorDefinition;
			Class[] strargsClass = new Class[] {String.class};
			Object[] strargs = new Object[] {args[3]};
			Constructor strargsConstructor;

			actorDefinition = Class.forName(args[2]);
			strargsConstructor = actorDefinition.getConstructor(strargsClass);
			actor = (Actor) createObject(strargsConstructor, strargs);

			runSimulation(actor, time, workloadfile);
	   		System.exit(0);
		} catch (Exception ex) {
			ex.printStackTrace();
	        System.exit(1);
	    }
    }
}
